;;; byte-test-minimal.el

(defconst byte-opt-test-patterns
  '(
    ;; next four are solved in bytecomp.el
    (let ((a most-positive-fixnum) (b 1) (c 1.0))  (+ a b c))
    (let ((a most-positive-fixnum) (b -2) (c 1.0)) (- a b c))
    (let ((a most-positive-fixnum) (b 2) (c 1.0))  (* a b c))
    (let ((a 3) (b 2) (c 1.0))                     (/ a b c))
    ;; to be soloved
    (let ((a (+ 1 (expt 2 -64))) (b (expt 2 -65))) (+ a -1 b))
    (let ((a (+ 1 (expt 2 -64))) (b (expt 2 -65))) (- a 1 (- b)))
    ;; thrown away
    (let ((a (expt 2 -1074)) (b 0.125))		   (* a 8 b))
    ;; to be soloved
    (let ((a 1.0))				   (* a 0))
    (let ((a 1.0))				   (* a 2.0 0))
    (let ((a 1.0))				   (/ 0 a))
    (let ((a 1.0))				   (/ 3 a 2))
    (let ((a most-positive-fixnum) (b 2.0))	   (* a 2 b))
    (let ((a 3) (b 2))				   (/ a b 1.0))
    (/ 3 -1)
    ;; new optimization
    (+ 4 3 2 1)
    (+ 4 3 2.0 1)
    (- 4 3 2 1)				; not new, for reference
    (- 4 3 2.0 1)			; not new, for reference
    (* 4 3 2 1)
    (* 4 3 2.0 1)
    (/ 4 3 2 1)
    (/ 4 3 2.0 1)
    (let ((a 3) (b 2))				   (+ a b 1))
    (let ((a 3) (b 2))				   (+ a b -1))
    (let ((a 3) (b 2))				   (- a b 1))
    (let ((a 3) (b 2))				   (- a b -1))
    (let ((a 3) (b 2))				   (+ a b a 1))
    (let ((a 3) (b 2))				   (+ a b a -1))
    (let ((a 3) (b 2))				   (- a b a 1))
    (let ((a 3) (b 2))				   (- a b a -1))
    (let ((a 3) (b 2))				   (* a b -1))
    (let ((a 3) (b 2))				   (* a -1))
    (let ((a 3) (b 2))				   (/ a b 1))
    (let ((a 3) (b 2))				   (/ (+ a b) 1))
    ))

(defun byte-opt-test-execute (&optional arg)
  (interactive "P")
  (let ((print-escape-nonascii t)
	(print-escape-newlines t)
	(print-quoted t)
	(buf "*Disassemble*")
	r1 r2
	o1 o2
	v0 v1 v2
	l1 l2
	indent-tabs-mode
	)
    (load-library "byte-opt")
    (mapc
     (lambda (pat)
       (push (byte-optimize-form pat) o1)
       (condition-case nil
	   (push (eval pat) v0)
	 (error (push nil v0)))
       (condition-case nil
           (push (funcall (byte-compile (list 'lambda nil pat))) v1)
	 (error (push nil v1)))
       (disassemble pat)
       (push (with-current-buffer buf
	       (setq buffer-read-only nil)
	       (untabify (point-min) (point-max)) (buffer-string))
	     r1))
     byte-opt-test-patterns)
    (let ((load-path '(".")))
      (load-library "byte-opt"))
    (mapc
     (lambda (pat)
       (push (byte-optimize-form pat) o2)
       (condition-case nil
           (push (funcall (byte-compile (list 'lambda nil pat))) v2)
	 (error (push nil v2)))
       (disassemble pat)
       (push (with-current-buffer buf (buffer-string)) r2))
     byte-opt-test-patterns)
    (setq r1 (nreverse r1)
	  r2 (nreverse r2)
	  o1 (nreverse o1)
	  o2 (nreverse o2)
	  v0 (nreverse v0)
	  v1 (nreverse v1)
	  v2 (nreverse v2))
    (with-output-to-temp-buffer "*BYTECOMP*"
      (with-current-buffer standard-output
	(mapc
	 (lambda (pat)
	   (let (col)
	     (insert (format "%s" pat))
	     (setq col (current-column))
	     (insert (format " --> %s\n" (pop v0)))
	     (insert (format "%s" (pop o1)))
	     (indent-to-column col)
	     (insert (format " --> %s\n" (pop v1)))
	     (insert (format "%s" (pop o2)))
	     (indent-to-column col)
	     (insert (format " --> %s\n" (pop v2)))
	     (setq l1 (split-string (pop r1) "\n")
		   l2 (split-string (pop r2) "\n"))
	     (if arg
		 (dotimes (i (max (length l1) (length l2)))
		   (insert (format "  %-32.32s %s\n"
				   (concat (car l1))
				   (concat (car l2))))
		   (setq l1 (cdr l1)
			 l2 (cdr l2)))
	       (insert "\n"))))
         byte-opt-test-patterns)
	(setq truncate-lines t)))))
